#!/usr/bin/env python

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import time
from raspisump import todaychart


def main():
    '''Initiate todaychart.py module to graph sump pit activity.'''
    csv_file = "/home/pi/raspi-sump/csv/waterlevel-{}.csv".format(
        time.strftime('%Y%m%d')
        )
    filename = "/home/pi/raspi-sump/charts/today.png"
    bytes2str = todaychart.bytesdate2str('%H:%M:%S')
    todaychart.graph(csv_file, filename, bytes2str)

if __name__ == "__main__":
    main()



'''Send Heartbeat alert to test that email notifications are working.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import os
import time
import smtplib
from datetime import datetime
try:
    import ConfigParser as configparser  # Python2
except ImportError:
    import configparser  # Python3
from collections import deque
import csv
from raspisump import log, alerts


config = configparser.RawConfigParser()

config.read('/home/pi/raspi-sump/raspisump.conf')

configs = {'email_to': config.get('email', 'email_to'),
           'email_from': config.get('email', 'email_from'),
           'smtp_authentication': config.getint(
           'email', 'smtp_authentication'),
           'smtp_tls': config.getint('email', 'smtp_tls'),
           'smtp_server': config.get('email', 'smtp_server'),
           'username': config.get('email', 'username'),
           'password': config.get('email', 'password')
           }

try:
    configs['heartbeat_interval'] = config.getint('email', 'heartbeat_interval')
except configparser.NoOptionError:
    configs['heartbeat_interval'] = 10080

def heartbeat_email_content():

    '''Build the contents of email body which will be sent as an alert'''

    time_of_day = alerts.current_time()
    hostname = alerts.host_name()
    
    subject = 'Subject: Raspi-Sump Heartbeat Notification' 
    message = 'Raspi-Sump Email Notifications Working'

    return "\r\n".join((
        "From: {}".format(configs['email_from']),
        "To: {}".format(configs['email_to']),
        "{}".format(subject),
        "",
        "{} - {} - {}.".format(hostname, time_of_day, message),
        "Next email test in {} minutes".format(configs['heartbeat_interval']),
        )
        )


def heartbeat_alerts():
    '''Send heartbeat email alert if water level greater than critical distance.'''
    recipients = configs['email_to'].split(', ')
    email_body = heartbeat_email_content()
    server = smtplib.SMTP(configs['smtp_server'])

    # Check if smtp server uses TLS
    if configs['smtp_tls'] == 1:
        server.starttls()
    else:
        pass
    # Check if smtp server uses authentication
    if configs['smtp_authentication'] == 1:
        username = configs['username']
        password = configs['password']
        server.login(username, password)
    else:
        pass

    server.sendmail(configs['email_from'], recipients, email_body)
    server.quit()


def determine_if_heartbeat():
    '''Determine if a heartbeat notification is required and if so, send
    the notification.'''

    heartbeat_interval_time = configs['heartbeat_interval']
    heartbeat_log = '/home/pi/raspi-sump/logs/heartbeat_log'
    if not os.path.isfile(heartbeat_log):
        heartbeat_alerts()
        log.log_heartbeat("Heartbeat Email Sent")

    else:
        with open(heartbeat_log, 'rt') as f:
            last_row = deque(csv.reader(f), 1)[0]
            last_email_sent = last_row[0]
            current_time = time.strftime('%Y-%m-%d %H:%M:%S')
            last_heartbeat_time = datetime.strptime(
                last_email_sent, '%Y-%m-%d %H:%M:%S'
            )
            time_now = datetime.strptime(current_time, '%Y-%m-%d %H:%M:%S')
            delta = (time_now - last_heartbeat_time)
            minutes_passed = int((delta).total_seconds() / 60)

            if minutes_passed >= heartbeat_interval_time:
                heartbeat_alerts()
                log.log_heartbeat("Heartbeat Email Sent")
            else:
                pass



'''Send SMTP email alerts in case of sump pump failure.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import os
import time
import smtplib
from datetime import datetime
import platform
try:
    import ConfigParser as configparser  # Python2
except ImportError:
    import configparser  # Python3
from collections import deque
import csv
from raspisump import log


config = configparser.RawConfigParser()

config.read('/home/pi/raspi-sump/raspisump.conf')

configs = {'email_to': config.get('email', 'email_to'),
           'email_from': config.get('email', 'email_from'),
           'smtp_authentication': config.getint(
               'email', 'smtp_authentication'),
           'smtp_tls': config.getint('email', 'smtp_tls'),
           'smtp_server': config.get('email', 'smtp_server'),
           'username': config.get('email', 'username'),
           'password': config.get('email', 'password'),
           'unit': config.get('pit', 'unit')
          }
# If item in raspisump.conf add to configs dict above.  If not then provide
# a default value
try:
    configs['alert_interval'] = config.getint('email', 'alert_interval')
except configparser.NoOptionError:
    configs['alert_interval'] = 5

try:
    configs['alert_when'] = config.get('pit', 'alert_when')
except configparser.NoOptionError:
    configs['alert_when'] = 'high'


def current_time():
    '''Return the current time as reported by the OS.'''
    return time.strftime('%I:%M%P %Z')

def host_name():
    '''Return the Raspberry Pi's Hostname'''
    return platform.node()

def unit_types():
    '''Determine  if inches or centimeters'''

    unit = configs['unit']

    if unit == 'imperial':
        return 'inches'
    if unit == 'metric':
        return 'centimeters'


def email_content(water_depth):
    '''Build the contents of email body which will be sent as an alert'''

    time_of_day = current_time()
    unit_type = unit_types()
    hostname = host_name()
    email_contents = {'subject_high': 'Subject: Sump Pump Alert!',
                      'subject_low': 'Subject: Low Water Level Alert!',
                      'message_high': 'Critical! The sump pit water level is',
                      'message_low': 'Warning! The waterlevel is down to'
                      }

    if configs['alert_when'] == 'high':
        subject = email_contents['subject_high']
        message = email_contents['message_high']
    else:
        subject = email_contents['subject_low']
        message = email_contents['message_low']

    return "\r\n".join((
        "From: {}".format(configs['email_from']),
        "To: {}".format(configs['email_to']),
        "{}".format(subject),
        "",
        "{} - {} - {} {} {}.".format(hostname, time_of_day, message, str(water_depth), unit_type),
        "Next alert in {} minutes".format(configs['alert_interval']),
        )
        )



def smtp_alerts(water_depth):
    '''Send email alert if water level greater than critical distance.'''
    recipients = configs['email_to'].split(', ')
    email_body = email_content(water_depth)
    server = smtplib.SMTP(configs['smtp_server'])

    # Check if smtp server uses TLS
    if configs['smtp_tls'] == 1:
        server.starttls()
    else:
        pass
    # Check if smtp server uses authentication
    if configs['smtp_authentication'] == 1:
        username = configs['username']
        password = configs['password']
        server.login(username, password)
    else:
        pass

    server.sendmail(configs['email_from'], recipients, email_body)
    server.quit()


def determine_if_alert(water_depth):
    '''Determine if an alert is required.  Only send if last alert has been
    sent more than the amount of time identified in the raspisump.conf file.
    Entry in conf file is alert_interval under the [email] section.'''

    alert_interval = configs['alert_interval']

    alert_log = '/home/pi/raspi-sump/logs/alert_log'

    if not os.path.isfile(alert_log):
        smtp_alerts(water_depth)
        log.log_alerts('Email SMS Alert Sent')

    else:
        with open(alert_log, 'rt') as f:
            last_row = deque(csv.reader(f), 1)[0]
            last_alert_sent = last_row[0]
            current_time = time.strftime('%Y-%m-%d %H:%M:%S')
            last_alert_time = datetime.strptime(
                last_alert_sent, '%Y-%m-%d %H:%M:%S'
            )
            time_now = datetime.strptime(current_time, '%Y-%m-%d %H:%M:%S')
            delta = (time_now - last_alert_time)
            minutes_passed = delta.seconds / 60
            
        if minutes_passed >= alert_interval:
            smtp_alerts(water_depth)
            log.log_alerts('Email SMS Alert Sent')

        else:
            pass



"""Graph sump pit activity."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import time
import numpy as np
import matplotlib as mpl

mpl.use("Agg")
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib import rcParams

rcParams.update({"figure.autolayout": True})
try:
    import ConfigParser as configparser  # Python2
except ImportError:
    import configparser  # Python3

config = configparser.RawConfigParser()
config.read("/home/pi/raspi-sump/raspisump.conf")
configs = {"unit": config.get("pit", "unit")}

try:
    configs["line_color"] = config.get("charts", "line_color")
except configparser.NoSectionError:
    configs["line_color"] = "FB921D"


MPL_VERSION = int(mpl.__version__.split(".")[0])  # Matplotlib major version

if MPL_VERSION > 1:
    rcParams[
        "date.autoformatter.hour"
    ] = "%H:%M:%S"  # Matplotlib 2.0 changed time formatting


def bytesdate2str(fmt, encoding="utf-8"):
    """Convert strpdate2num from bytes to string as required in Python3.

    This is a workaround as described in the following tread;
    https://github.com/matplotlib/matplotlib/issues/4126/

    Credit to github user cimarronm for this workaround.
    """

    strconverter = mdates.strpdate2num(fmt)

    def bytesconverter(b):
        s = b.decode(encoding)
        return strconverter(s)

    return bytesconverter


def graph(csv_file, filename, bytes2str):
    """Create a line graph from a two column csv file."""

    unit = configs["unit"]
    date, value = np.loadtxt(
        csv_file, delimiter=",", unpack=True, converters={0: bytes2str}
    )
    fig = plt.figure(figsize=(10, 3.5))

    # axisbg is deprecated in matplotlib 2.x. Maintain 1.x compatibility
    if MPL_VERSION > 1:
        fig.add_subplot(111, facecolor="white", frameon=False)
    else:
        fig.add_subplot(111, axisbg="white", frameon=False)

    rcParams.update({"font.size": 9})
    plt.plot_date(
        x=date,
        y=value,
        ls="solid",
        linewidth=2,
        color="#" + configs["line_color"],
        fmt=":",
    )
    title = "Sump Pit Water Level {}".format(time.strftime("%Y-%m-%d %H:%M"))
    title_set = plt.title(title)
    title_set.set_y(1.09)
    plt.subplots_adjust(top=0.86)

    if unit == "imperial":
        plt.ylabel("inches")
    if unit == "metric":
        plt.ylabel("centimeters")

    plt.xlabel("Time of Day")
    plt.xticks(rotation=30)
    plt.grid(True, color="#ECE5DE", linestyle="solid")
    plt.tick_params(axis="x", bottom="off", top="off")
    plt.tick_params(axis="y", left="off", right="off")
    plt.savefig(filename, dpi=72)



''' Module to take a water_level reading.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

try:
    import ConfigParser as configparser  # Python2
except ImportError:
    import configparser  # Python3
from hcsr04sensor import sensor
from raspisump import log, alerts, heartbeat

config = configparser.RawConfigParser()
config.read('/home/pi/raspi-sump/raspisump.conf')

configs = {'critical_water_level': config.getint('pit', 'critical_water_level'),
           'pit_depth': config.getint('pit', 'pit_depth'),
           'temperature': config.getint('pit', 'temperature'),
           'trig_pin': config.getint('gpio_pins', 'trig_pin'),
           'echo_pin': config.getint('gpio_pins', 'echo_pin'),
           'unit': config.get('pit', 'unit')
           }

# If item in raspisump.conf add to configs dict. If not provide defaults.
try:
    configs['alert_when'] = config.get('pit', 'alert_when')
except configparser.NoOptionError:
    configs['alert_when'] = 'high'

try:
    configs['heartbeat'] = config.getint('email', 'heartbeat')
except configparser.NoOptionError:
    configs['heartbeat'] = 0


def initiate_heartbeat():
    '''Initiate the heartbeat email process if needed'''
    if configs['heartbeat'] == 1:
        heartbeat.determine_if_heartbeat()
    else:
        pass


def water_reading():
    '''Initiate a water level reading.'''
    pit_depth = configs['pit_depth']
    trig_pin = configs['trig_pin']
    echo_pin = configs['echo_pin']
    round_to = 1
    temperature = configs['temperature']
    unit = configs['unit']

    value = sensor.Measurement(trig_pin, echo_pin, temperature, unit, round_to)
    
    try:
        raw_distance = value.raw_distance(sample_wait=0.3)
    except SystemError:
        log.log_errors("**ERROR - Signal not received. Possible cable or sensor problem.")
        exit(0) 
    
    if unit == 'imperial':
        return value.depth_imperial(raw_distance, pit_depth)
    if unit == 'metric':
        return value.depth_metric(raw_distance, pit_depth)


def water_depth():
    '''Determine the depth of the water, log result and generate alert
    if needed.
    '''

    critical_water_level = configs['critical_water_level']
    water_depth = water_reading()


    if water_depth < 0.0:
        water_depth = 0.0
    log.log_reading(water_depth)
    
    if water_depth > critical_water_level and configs['alert_when'] == 'high':
        alerts.determine_if_alert(water_depth)
    elif water_depth < critical_water_level and configs['alert_when'] == 'low':
        alerts.determine_if_alert(water_depth)
    else:
        pass

    initiate_heartbeat()



from setuptools import setup
import os
version = '1.2.1'

homedir = '/home/pi/raspi-sump/'

if os.path.isfile(homedir + 'raspisump.conf'):
    cmd = 'cp -u ' + homedir + 'raspisump.conf ' + homedir + \
        'raspisump.conf.save'
    os.system(cmd)

raspi_sump_files = ['bin/rsump.py',
                    'bin/rsumpchart.py',
                    'bin/rsumpmonitor.py',
                    'bin/rsumpwebchart.py'
                    ]

add_files = [(homedir + 'sample_config', ['conf/raspisump.conf']),
             (homedir + 'sample_config', ['conf/lighttpd.conf']),
             (homedir + 'csv', ['conf/csv/README.md']),
             (homedir + 'logs', ['conf/logs/README.md']),
             (homedir + 'charts', ['conf/charts/README.md']),
             (homedir + 'docs', ['docs/README.md']),
             (homedir + 'docs', ['docs/install.md']),
             (homedir + 'cron', ['cron/README.md']),
             (homedir + 'cron', ['cron/picrontab']),
             (homedir + 'web', ['conf/web/index.html']),
             (homedir + 'web/images', ['conf/web/images/logo.png']),
             (homedir + 'web/css', ['conf/web/css/index.html']),
             (homedir + 'web/css', ['conf/web/css/raspi.css'])
             ]

setup(name='raspisump',
      version=version,
      description='A sump pit monitoring system for Raspberry Pi',
      long_description=open("./README.md", "r").read(),
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Environment :: Console",
          "Intended Audience :: End Users/Desktop",
          "Natural Language :: English",
          "Operating System :: POSIX :: Linux",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.2",
          "Programming Language :: Python :: 3.4",
          "Topic :: Home Automation",
          "License :: OSI Approved :: MIT License",
          ],
      author='Al Audet',
      author_email='alaudet@linuxnorth.org',
      url='http://www.linuxnorth.org/raspi-sump/',
      download_url='https://github.com/alaudet/raspi-sump/releases',
      license='MIT License',
      packages=['raspisump'],
      scripts=raspi_sump_files,
      data_files=add_files,
      install_requires=['hcsr04sensor']
      )

if os.path.isdir(homedir):
    cmd = 'chown -R pi ' + homedir
    os.system(cmd)
    cmd = 'chmod 600 ' + homedir + 'raspisump.conf'
    os.system(cmd)

